#include "TetrisBlock.h"

CTetrisBlock::CTetrisBlock()
{
	int num_blocks_x = WINDOW_WIDTH / (BLOCK_SIZE + BLOCK_SPACING);
	int num_blocks_y = WINDOW_HEIGHT / (BLOCK_SIZE + BLOCK_SPACING);

	m_iPosX = num_blocks_x / 2;
	m_iPosY = num_blocks_y - 1;
}

CTetrisBlock::~CTetrisBlock()
{
	Destroy();	
}

void CTetrisBlock::Create()
{

}

void CTetrisBlock::Draw()
{
	tRect quad;
	quad.m_iLeft = m_iPosX * (BLOCK_SIZE + BLOCK_SPACING) + BLOCK_SPACING;
	quad.m_iRight = quad.m_iLeft + BLOCK_SIZE - BLOCK_SPACING;
	quad.m_iTop = m_iPosY * (BLOCK_SIZE + BLOCK_SPACING) - BLOCK_SPACING;
	quad.m_iBottom = quad.m_iTop - BLOCK_SIZE + BLOCK_SPACING;

	glColor3d(1,1,1);
	glBegin(GL_QUADS);
	glVertex3f(quad.m_iLeft, quad.m_iBottom, 0);
	glVertex3f(quad.m_iRight, quad.m_iBottom, 0);
	glVertex3f(quad.m_iRight, quad.m_iTop, 0);
	glVertex3f(quad.m_iLeft, quad.m_iTop, 0);
	glEnd();
}

bool CTetrisBlock::Rotate(int dir, int origin_x, int origin_y, bool can_rotate_test)
{
	int next_x_pos, next_y_pos;

	int offset_x = m_iPosX - origin_x;
	int offset_y = m_iPosY - origin_y;
	
	if(offset_x ==0 && offset_y == 0)
		return true;

	int X = offset_y;
	int Y = -offset_x;

	next_x_pos = origin_x + X;
	next_y_pos = origin_y + Y;

	if(can_rotate_test)
	{
		int num_blocks_x = WINDOW_WIDTH / (BLOCK_SIZE + BLOCK_SPACING);
		int num_blocks_y = WINDOW_HEIGHT / (BLOCK_SIZE + BLOCK_SPACING);

		if(next_y_pos == 0 || next_y_pos == num_blocks_y)
			return false;
		if(next_x_pos == -1|| next_x_pos == num_blocks_x)
			return false;
	}
	else
	{
		m_iPosX = next_x_pos;
		m_iPosY = next_y_pos;
	}

	return true;
}

bool CTetrisBlock::Move(int dir, bool can_move_test)
{
	if(can_move_test)
	{
		int num_blocks_x = WINDOW_WIDTH / (BLOCK_SIZE + BLOCK_SPACING);
		int num_blocks_y = WINDOW_HEIGHT / (BLOCK_SIZE + BLOCK_SPACING);

		int next_x_pos = 0, next_y_pos = 0;
		switch(dir)
		{
			case DISPLACE_DOWN:
			{
				next_y_pos = m_iPosY - 1;
				next_x_pos = m_iPosX;
			}
			break;

			case DISPLACE_LEFT:
			{
				next_x_pos = m_iPosX - 1;
				next_y_pos = m_iPosY;
			}
			break;

			case DISPLACE_RIGHT:
			{
				next_x_pos = m_iPosX + 1;
				next_y_pos = m_iPosY;
			}
			break;
		}

		if(next_y_pos == 0 || next_y_pos == num_blocks_y)
			return false;
		if(next_x_pos == -1|| next_x_pos == num_blocks_x)
			return false;

		return true;
	}
	else
	{
		switch(dir)
		{
			case DISPLACE_DOWN:
				m_iPosY = m_iPosY - 1;
			break;

			case DISPLACE_LEFT:
				m_iPosX = m_iPosX - 1;
			break;

			case DISPLACE_RIGHT:
				m_iPosX = m_iPosX + 1;
			break;
		}
	}

	return true;
}

void CTetrisBlock::Destroy()
{

}

void CTetrisBlock::SetPosX(int x)
{
	m_iPosX = x;
}

void CTetrisBlock::SetPosY(int y)
{
	m_iPosY = y;
}

int CTetrisBlock::GetPosX()
{
	return m_iPosX;
}

int CTetrisBlock::GetPosY()
{
	return m_iPosY;
}