#include "TetrisFigure.h"

CTetrisFigure::CTetrisFigure()
{
	m_ppBlocks = NULL;
	m_iBlocksCount = 0;
}

CTetrisFigure::~CTetrisFigure()
{
	Destroy();
}

void CTetrisFigure::Create(int num_blocks)
{
	if(m_ppBlocks != NULL)
		Destroy();
	
	if(num_blocks > 0)
	{
		m_iBlocksCount = num_blocks;

		m_ppBlocks = new CTetrisBlock*[num_blocks];
		for (int i=0; i<num_blocks; i++)
			m_ppBlocks[i] = new CTetrisBlock;
	}
}

void CTetrisFigure::Destroy()
{
	if(m_ppBlocks)
	{
		for (int i=0; i<m_iBlocksCount; i++)
		{
			if(m_ppBlocks[i] !=NULL)
				delete m_ppBlocks[i];
		}
		delete[] m_ppBlocks;
	}

	m_ppBlocks = NULL;
}

void CTetrisFigure::Draw()
{
	if(!m_ppBlocks)
		return;

	for (int i=0; i < GetBlockCount(); i++)
	{
		if(m_ppBlocks[i] !=NULL)
			m_ppBlocks[i]->Draw();
	}
}

/***************************** NEW !!!!! ************************************************/
/*	Here we do rotation and rotation tests for each of the block. If each and every block
	can be rotated, return true, otherwise return false.
*/
bool CTetrisFigure::Rotate(int dir, bool can_rotate_test)
{
	if(can_rotate_test)
	{
		for (int i=0; i<GetBlockCount(); i++)
		{
			if(m_ppBlocks[i] !=NULL)
				if(!m_ppBlocks[i]->Rotate(dir, m_ppBlocks[0]->GetPosX(), m_ppBlocks[0]->GetPosY(), true))
					return false;
		}
	}
	else
	{
		for (int i=0; i<GetBlockCount(); i++)
		{
			if(m_ppBlocks[i] !=NULL)
				m_ppBlocks[i]->Rotate(dir, m_ppBlocks[0]->GetPosX(), m_ppBlocks[0]->GetPosY(), false);
		}
	}

	
	return true;
}
/***************************** NEW !!!!! ************************************************/

/***************************** NEW !!!!! ************************************************/
/*	Here we do movement and movement tests for each of the block. If each and every block
	can be moved, return true, otherwise return false.
*/
bool CTetrisFigure::Move(int dir, bool can_move_test)
{
	// We do testing
	if(can_move_test)
	{
		for (int i=0; i<GetBlockCount(); i++)
		{
			if(m_ppBlocks[i] !=NULL)
			{
				if(!m_ppBlocks[i]->Move(dir, true))
					return false;
			}
		}
	}
	else
	{
		for (int i=0; i<GetBlockCount(); i++)
		{
			if(m_ppBlocks[i] !=NULL)
				m_ppBlocks[i]->Move(dir, false);
		}
	}

	return true;
}
/***************************** NEW !!!!! ************************************************/

CTetrisBlock *CTetrisFigure::GetBlock(int x, int y)
{
	for (int i=0; i<GetBlockCount(); i++)
	{
		if(m_ppBlocks[i] !=NULL)
		{
			if(m_ppBlocks[i]->GetPosX() == x && m_ppBlocks[i]->GetPosY() == y)
				return m_ppBlocks[i];
		}
	}

	return NULL;
}

CTetrisBlock *CTetrisFigure::GetBlock(int index)
{
	if(index >=0 && index < GetBlockCount())
		return m_ppBlocks[index];

	return NULL;
}

int CTetrisFigure::GetBlockCount()
{
	return m_iBlocksCount;
}


