#include "TetrisGame.h"

// IF ANY OF ANIMATED BLOCKS ARE LYING ON THE ELIMINATING LINE, MOVE THEM OUT OF THERE (PICK UP ANOTHER ONE)
CTetrisGame::CTetrisGame()
{
	m_pCurrentFigure = NULL;
	m_pNextFigure = NULL;
	m_ppTetrisMatrix = NULL;

	m_eGameState = TetrisGame_Normal;
	m_iScore = 0;
	m_iFullLine = -1;
	m_fLastTime = 0;
	
}

CTetrisGame::~CTetrisGame()
{
	Destroy();
}


void CTetrisGame::Destroy()
{
	if(m_pCurrentFigure !=NULL)
	{
		delete m_pCurrentFigure;
		m_pCurrentFigure = NULL;
	}

	int num_blocks_x = WINDOW_WIDTH / (BLOCK_SIZE + BLOCK_SPACING);
	int num_blocks_y = WINDOW_HEIGHT / (BLOCK_SIZE + BLOCK_SPACING);

	if(m_pTemplateBlocks)
	{
		delete[] m_pTemplateBlocks;
		m_pTemplateBlocks = NULL;
	}

	if(m_pTemplateFigures)
	{
		delete[] m_pTemplateFigures;
		m_pTemplateFigures = NULL;
	}


	m_pOldFigures.erase();

	if(m_ppTetrisMatrix)
	{
		for (int y=0; y<num_blocks_y; y++)
		{
			delete[] m_ppTetrisMatrix[y];
			m_ppTetrisMatrix[y] = NULL;
		}

		delete[] m_ppTetrisMatrix;
		m_ppTetrisMatrix = NULL;

	}

	m_iScore = 0;
}

/************************ NEW !!!!! *********************************************/
/*
	Our Update() function. Here we do all checks vs game state, eliminate full 
	lines, and do some other stuff.
*/
void CTetrisGame::Update()
{
	if(m_eGameState == TetrisGame_Paused)
		return;

	int num_blocks_x = WINDOW_WIDTH / (BLOCK_SIZE + BLOCK_SPACING);
	int num_blocks_y = WINDOW_HEIGHT / (BLOCK_SIZE + BLOCK_SPACING);

	if(m_eGameState == TetrisGame_Normal)
	{
		double CurrentTime = m_Timer.getAbsoluteTime();
		if(CurrentTime - m_fLastTime >= TIME_AQUIRE_USER_INPUT) // If the time given to aquire user input has passed
		{
			if(Move(DISPLACE_DOWN, true) == false) // Our figure cannot move down anymore, map it to game blocks array
			{
				for (int i=0; i<num_blocks_y; i++)
				{
					if(LineIsFull(i))
					{
						m_iFullLine = i;
						m_eGameState = TetrisGame_EliminatingLine;
						break;
					}
				}

				if(m_iFullLine == -1)
					DropRandomFigure();
			}
			else
				Move(DISPLACE_DOWN, false);	// Our figure CAN be moved down, so move it now

			m_fLastTime = CurrentTime;
		}
	}
		
	// If there are any full lines (from the check above) - animate that line until the end
	if(m_eGameState == TetrisGame_EliminatingLine)
	{
		if(!AnimateLine(m_iFullLine)) // If AnimateLine returns false, it means that all blocks have finished their current animation
		{
			// Line blocks animations are complete, remove this line from game blocks array
			for (int i=0; i<num_blocks_x; i++)
				m_ppTetrisMatrix[m_iFullLine][i] = NULL;

			// Shift above lines down 
			ShiftLines(m_iFullLine);
			
			// Restore memeber variables for next check
			m_eGameState = TetrisGame_Normal;
			m_iFullLine = -1;
		}
	}
}

/************************ NEW !!!!! *********************************************/

void CTetrisGame::Draw()
{
	/************************ NEW !!!!! *********************************************/
	/*
	Here we call our Update() function that manages all updates to the game
	*/

	Update();
	/************************ NEW !!!!! *********************************************/
	
	int num_blocks_x = WINDOW_WIDTH / (BLOCK_SIZE + BLOCK_SPACING);
	int num_blocks_y = WINDOW_HEIGHT / (BLOCK_SIZE + BLOCK_SPACING);

	for (int i=0; i<num_blocks_y; i++)
	{
		for (int j=0; j<num_blocks_x; j++)
		{
			if(m_ppTetrisMatrix[i][j] !=NULL)
				m_ppTetrisMatrix[i][j]->Draw();
		}
	}
}

void CTetrisGame::Create()
{
	int num_blocks_x = WINDOW_WIDTH / (BLOCK_SIZE + BLOCK_SPACING);
	int num_blocks_y = WINDOW_HEIGHT / (BLOCK_SIZE + BLOCK_SPACING);
	
	m_ppTetrisMatrix = new CTetrisBlock**[num_blocks_y];

	for (int i=0; i<num_blocks_y; i++)
	{
		m_ppTetrisMatrix[i] = new CTetrisBlock*[num_blocks_x];
		for (int j=0; j<num_blocks_x; j++)
			m_ppTetrisMatrix[i][j] = NULL;
	}
}

/*********************** NEW !!!!! *******************************************/
/*
	Here we move the current figure (since its the only one the user can control)
	and also test if it can be moved in specified direction against window 
	boundaries and game blocks array.
*/
bool CTetrisGame::Move(int dir, bool can_move_test)
{
	if(m_pCurrentFigure == NULL)
		return false;

	if(can_move_test)
	{
		if(!m_pCurrentFigure ->Move(dir, true))
			return false;
		else			
		{
			CTetrisFigure tmp;
			tmp.Create(m_pCurrentFigure->GetBlockCount());

			for (int i=0; i<m_pCurrentFigure->GetBlockCount(); i++)
			{
				tmp.GetBlock(i)->SetPosX(m_pCurrentFigure->GetBlock(i)->GetPosX());
				tmp.GetBlock(i)->SetPosY(m_pCurrentFigure->GetBlock(i)->GetPosY());
			}

			tmp.Move(dir, false);
			
			bool all_good = true;
			for (i=0; i<tmp.GetBlockCount(); i++)
			{
				int block_at_x = tmp.GetBlock(i)->GetPosX();
				int block_at_y = tmp.GetBlock(i)->GetPosY();
				
				if(m_pCurrentFigure ->GetBlock(block_at_x, block_at_y) == NULL)
				{
					if(m_ppTetrisMatrix[block_at_y][block_at_x] != NULL)
						return false;
				}
				
			}
		}
	}
	else
	{
		MapFigure (m_pCurrentFigure , 0);
		m_pCurrentFigure->Move(dir, false);
		MapFigure(m_pCurrentFigure , 1);
	}

	return true;
}
/*********************** NEW !!!!! *******************************************/


bool CTetrisGame::Rotate(int dir, bool can_rotate_test)
{
	if(can_rotate_test)
	{
		if(!m_pCurrentFigure->Rotate(dir, true)) 
		return false;

		CTetrisFigure tmp;
		tmp.Create(m_pCurrentFigure->GetBlockCount());

		for (int i=0; i<m_pCurrentFigure->GetBlockCount(); i++)
		{
			tmp.GetBlock(i)->SetPosX(m_pCurrentFigure->GetBlock(i)->GetPosX());
			tmp.GetBlock(i)->SetPosY(m_pCurrentFigure->GetBlock(i)->GetPosY());
		}

		tmp.Rotate(dir, false);
 
		for (i=0; i<tmp.GetBlockCount(); i++)
		{
			int block_at_x = tmp. GetBlock(i)->GetPosX();
			int block_at_y = tmp. GetBlock(i)->GetPosY();
					
			if(m_pCurrentFigure->GetBlock(block_at_x, block_at_y) == NULL)
			{
				if(m_ppTetrisMatrix[block_at_y][block_at_x] != NULL)
						return false;
			}
		}
	}
	else
	{
		MapFigure(m_pCurrentFigure, 0);
		m_pCurrentFigure->Rotate(dir, false);
		MapFigure (m_pCurrentFigure, 1);
	}

	return true;
}

void CTetrisGame::MapFigure(CTetrisFigure *figure, bool assign)
{
	// To see the current figure rotate and move, we add this only like to see how it works
	// It will be removed later...
	if(figure ==NULL)
		return;

	for (int i=0; i<figure->GetBlockCount(); i++)
	{
		if(assign)
		{
			if(figure->GetBlock(i) !=NULL)	
				m_ppTetrisMatrix[figure->GetBlock(i)->GetPosY()][figure->GetBlock(i)->GetPosX()] = figure->GetBlock(i);
		}
		else
		{
			if(figure->GetBlock(i) !=NULL)	
				m_ppTetrisMatrix[figure->GetBlock(i)->GetPosY()][figure->GetBlock(i)->GetPosX()] = NULL;
		}
	}
}

void CTetrisGame::DropRandomFigure()
{
	if(m_pCurrentFigure !=NULL)
		m_pOldFigures.push_back(m_pCurrentFigure);

	CTetrisFigure *figure = new CTetrisFigure;
	figure->Create(4);

	int num_blocks_x = WINDOW_WIDTH / (BLOCK_SIZE + BLOCK_SPACING);
	int num_blocks_y = WINDOW_HEIGHT / (BLOCK_SIZE + BLOCK_SPACING);

	figure->GetBlock(0)->SetPosX(num_blocks_x / 2);
	figure->GetBlock(0)->SetPosY(num_blocks_y - 2);
	
	figure->GetBlock(1)->SetPosX((num_blocks_x / 2) + 1);
	figure->GetBlock(1)->SetPosY(num_blocks_y - 2);

	figure->GetBlock(2)->SetPosX((num_blocks_x / 2) + 1);
	figure->GetBlock(2)->SetPosY(num_blocks_y - 1);

	figure->GetBlock(3)->SetPosX(num_blocks_x / 2);
	figure->GetBlock(3)->SetPosY(num_blocks_y - 1);

	m_pCurrentFigure = figure;
	MapFigure(m_pCurrentFigure, true);
}

bool CTetrisGame::LineIsFull(unsigned int line_index)
{
	int num_blocks_x = WINDOW_WIDTH / (BLOCK_SIZE + BLOCK_SPACING);
	
	for (int x=0; x<num_blocks_x; x++)
	{
		if(m_ppTetrisMatrix[line_index][x] == NULL)
			return false;
	}

	return true;
}

void CTetrisGame::ShiftLines(unsigned int line_eliminated)
{
	int num_blocks_x = WINDOW_WIDTH / (BLOCK_SIZE + BLOCK_SPACING);
	int num_blocks_y = WINDOW_HEIGHT / (BLOCK_SIZE + BLOCK_SPACING);

	for (int i=line_eliminated+1; i<num_blocks_y; i++) // Remeber, we are shifting lines that are above the eliminated one
	{
		// Shift this line down
		for (int j=0; j<num_blocks_x; j++)
		{
			// Move line down with coordinates
			if(m_ppTetrisMatrix[i][j] !=NULL)
				m_ppTetrisMatrix[i][j]->Move(DISPLACE_DOWN, false);

			// Move line down in pointers
			m_ppTetrisMatrix[i-1][j] = m_ppTetrisMatrix[i][j];
			// Eliminate moved line in pointers
			m_ppTetrisMatrix[i][j] = NULL;
		}
	}		
}

bool CTetrisGame::AnimateLine(unsigned int line_index)
{

	return false;
}
