#ifndef TETRIS_GAME_INCLUDED
#define TETRIS_GAME_INCLUDED

#include "TetrisFigure.h"
#include "Timer.h"


enum eTetrisGameState
{
	TetrisGame_Normal,
	TetrisGame_AquireUserInput,
	TetrisGame_Paused,
	TetrisGame_EliminatingLine
};

class CTetrisGame
{
public:
		
	void Create();
	void Draw();
	void Destroy();
		
	void Update();

	bool Rotate(int dir, bool can_rotate_test);
	bool Move(int dir, bool can_move_test);

	void DropRandomFigure();
	void MapFigure(CTetrisFigure *figure, bool assign);

	/****************** NEW !!!!! ***********************************************/
	/* This function will handle the animation of the given line */
	bool AnimateLine(unsigned int line_index);

	/* This function will shift all lines from above downwards*/
	void ShiftLines(unsigned int line_eliminated);
	
	/* This function returns true if the given line is full, false otherwise*/
	bool LineIsFull(unsigned int line_index);
	/****************** NEW !!!!! ***********************************************/

	CTetrisGame();
	virtual ~CTetrisGame();
	
private:

	/****************** NEW !!!!! ***********************************************/
	/* 
		Adding a member variable of CTimer class, 
		we will be able to retrieve current time.
		*/
	CTimer m_Timer;
	double m_fLastTime;
	
	/* This variables are going to watch after:*/
	eTetrisGameState m_eGameState;	// Current game state - paused, normal ,etc
	int m_iFullLine;				// A full line index

	/****************** NEW !!!!! ***********************************************/
	int m_iScore;
	
	CTetrisFigure *m_pTemplateFigures;
	CTetrisBlock *m_pTemplateBlocks;

	int m_iNumTemplateFigures;
	int m_iNumTemplateBlocks;

	CTetrisBlock ***m_ppTetrisMatrix;
	
	CTetrisFigure *m_pCurrentFigure;
	CTetrisFigure *m_pNextFigure;

	CDoubleList<CTetrisFigure *> m_pOldFigures;
};

#endif